/**************************************************************/
/*     File: C_main.c                                         */
/*  Purpose: Main Application Code                            */
/**************************************************************/

/*
This is a simple scatter-loaded application which runs under the 
ARMulator (or the ARM PID board).
It simply displays the linker-generated scatter symbols on the screen.
It is not normally necessary to access these linker symbols in application 
code (only really needed in init code) - it is done here just for illustration.

The C entry point is call 'C_Entry()' not 'main()' to prevent the ANSI C 
libraries being pulled in during the link step, because the Embedded C 
libraries are being used here instead.

The Embedded C libraries do not contain printf(), so here Angel SWIs 
together with sprintf() are used to display text onto the console.  
This mechanism is portable across ARMulator, Angel, EmbeddedICE and Multi-ICE.
*/

#include <stdio.h>

#ifdef __thumb
/* Define Angel Semihosting SWI to be Thumb one */
#define SemiSWI 0xAB
#else
/* Define Angel Semihosting SWI to be ARM one */
#define SemiSWI 0x123456
#endif

/* Write a string */
__swi(SemiSWI) void _Write0(unsigned op, char *string);
#define Write0(string) _Write0 (0x4,string)

/* Exit */
__swi(SemiSWI) void _Exit(unsigned op, unsigned except);
#define Exit() _Exit (0x18,0x20026)

extern void C_func(void);

/*
The following symbols are generated by the linker.  They are imported 
WEAKly because they may not all have defined values. Those which are
undefined will take the value zero.
*/

extern ___weak void Load$$32bitRAM$$Base(void);
extern ___weak void Image$$32bitRAM$$Base(void);
extern ___weak void Image$$32bitRAM$$Length(void);
extern ___weak void Image$$32bitRAM$$ZI$$Base(void);
extern ___weak void Image$$32bitRAM$$ZI$$Length(void);

char buf[40];   /* will consume some ZI data */

void C_Entry(void)
{
  sprintf(buf, "Load$$32bitRAM$$Base = %p\n", Load$$32bitRAM$$Base);
  Write0(buf);
  sprintf(buf, "Image$$32bitRAM$$Base = %p\n", Image$$32bitRAM$$Base);
  Write0(buf);
  sprintf(buf, "Image$$32bitRAM$$Length = %p\n", Image$$32bitRAM$$Length);
  Write0(buf);
  sprintf(buf, "Image$$32bitRAM$$ZI$$Base = %p\n", Image$$32bitRAM$$ZI$$Base);
  Write0(buf);
  sprintf(buf, "Image$$32bitRAM$$ZI$$Length = %p\n", Image$$32bitRAM$$ZI$$Length);
  Write0(buf);

  C_func();

  Exit();
}

